@@ -57,11 +57,11 @@ import tempfile
 import threading
 
 import eventlet
+from eventlet import patcher
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import importutils
-import six
 
 from oslo_privsep._i18n import _
 from oslo_privsep import capabilities
@@ -75,6 +75,25 @@ if platform.system() == 'Linux':
 LOG = logging.getLogger(__name__)
 
 
+EVENTLET_MODULES = ('os', 'select', 'socket', 'thread', 'time', 'MySQLdb',
+                    'builtins', 'subprocess')
+EVENTLET_LIBRARIES = []
+
+
+def _null():
+    return []
+
+
+for module in EVENTLET_MODULES:
+    if hasattr(patcher, '_green_%s_modules' % module):
+        method = getattr(patcher, '_green_%s_modules' % module)
+    elif hasattr(patcher, '_green_%s' % module):
+        method = getattr(patcher, '_green_%s' % module)
+    else:
+        method = _null()
+    EVENTLET_LIBRARIES.append((module, method))
+
+
 @enum.unique
 class StdioFd(enum.IntEnum):
     # NOTE(gus): We can't use sys.std*.fileno() here.  sys.std*
@@ -89,17 +108,6 @@ class StdioFd(enum.IntEnum):
     STDERR = 2
 
 
-@enum.unique
-class Message(enum.IntEnum):
-    """Types of messages sent across the communication channel"""
-    PING = 1
-    PONG = 2
-    CALL = 3
-    RET = 4
-    ERR = 5
-    LOG = 6
-
-
 class FailedToDropPrivileges(Exception):
     pass
 
@@ -167,35 +175,38 @@ class PrivsepLogHandler(pylogging.Handler):
         data['msg'] = record.getMessage()
         data['args'] = ()
 
-        self.channel.send((None, (Message.LOG, data)))
+        self.channel.send((None, (comm.Message.LOG, data)))
 
 
 class _ClientChannel(comm.ClientChannel):
     """Our protocol, layered on the basic primitives in comm.ClientChannel"""
 
-    def __init__(self, sock):
+    def __init__(self, sock, context):
+        self.log = logging.getLogger(context.conf.logger_name)
         super(_ClientChannel, self).__init__(sock)
         self.exchange_ping()
 
     def exchange_ping(self):
         try:
             # exchange "ready" messages
-            reply = self.send_recv((Message.PING.value,))
-            success = reply[0] == Message.PONG
+            reply = self.send_recv((comm.Message.PING.value,))
+            success = reply[0] == comm.Message.PONG
         except Exception as e:
-            LOG.exception('Error while sending initial PING to privsep: %s', e)
+            self.log.exception('Error while sending initial PING to privsep: '
+                               '%s', e)
             success = False
         if not success:
             msg = _('Privsep daemon failed to start')
-            LOG.critical(msg)
+            self.log.critical(msg)
             raise FailedToDropPrivileges(msg)
 
-    def remote_call(self, name, args, kwargs):
-        result = self.send_recv((Message.CALL.value, name, args, kwargs))
-        if result[0] == Message.RET:
+    def remote_call(self, name, args, kwargs, timeout):
+        result = self.send_recv((comm.Message.CALL.value, name, args, kwargs),
+                                timeout)
+        if result[0] == comm.Message.RET:
             # (RET, return value)
             return result[1]
-        elif result[0] == Message.ERR:
+        elif result[0] == comm.Message.ERR:
             # (ERR, exc_type, args)
             #
             # TODO(gus): see what can be done to preserve traceback
@@ -206,16 +217,16 @@ class _ClientChannel(comm.ClientChannel):
             raise ProtocolError(_('Unexpected response: %r') % result)
 
     def out_of_band(self, msg):
-        if msg[0] == Message.LOG:
+        if msg[0] == comm.Message.LOG:
             # (LOG, LogRecord __dict__)
             message = {encodeutils.safe_decode(k): v
                        for k, v in msg[1].items()}
             record = pylogging.makeLogRecord(message)
-            if LOG.isEnabledFor(record.levelno):
-                LOG.logger.handle(record)
+            if self.log.isEnabledFor(record.levelno):
+                self.log.logger.handle(record)
         else:
-            LOG.warning('Ignoring unexpected OOB message from privileged '
-                        'process: %r', msg)
+            self.log.warning('Ignoring unexpected OOB message from privileged '
+                             'process: %r', msg)
 
 
 def fdopen(fd, *args, **kwargs):
@@ -258,6 +269,21 @@ def replace_logging(handler, log_root=None):
     log_root.addHandler(handler)
 
 
+def un_monkey_patch():
+    for eventlet_mod_name, func_modules in EVENTLET_LIBRARIES:
+        if not eventlet.patcher.is_monkey_patched(eventlet_mod_name):
+            continue
+
+        for name, mod in func_modules():
+            patched_mod = sys.modules.get(name)
+            orig_mod = eventlet.patcher.original(name)
+            for attr_name in mod.__patched__:
+                patched_attr = getattr(mod, attr_name, None)
+                unpatched_attr = getattr(orig_mod, attr_name, None)
+                if patched_attr is not None:
+                    setattr(patched_mod, attr_name, unpatched_attr)
+
+
 class ForkingClientChannel(_ClientChannel):
     def __init__(self, context):
         """Start privsep daemon using fork()
@@ -279,6 +305,7 @@ class ForkingClientChannel(_ClientChannel):
 
         if os.fork() == 0:
             # child
+            un_monkey_patch()
 
             channel = comm.ServerChannel(sock_b)
             sock_a.close()
@@ -294,7 +321,7 @@ class ForkingClientChannel(_ClientChannel):
         # parent
 
         sock_b.close()
-        super(ForkingClientChannel, self).__init__(sock_a)
+        super(ForkingClientChannel, self).__init__(sock_a, context)
 
 
 class RootwrapClientChannel(_ClientChannel):
@@ -344,7 +371,7 @@ class RootwrapClientChannel(_ClientChannel):
                     raise
             os.rmdir(tmpdir)
 
-        super(RootwrapClientChannel, self).__init__(sock)
+        super(RootwrapClientChannel, self).__init__(sock, context)
 
 
 class Daemon(object):
@@ -432,11 +459,11 @@ class Daemon(object):
         :return: A tuple of the return status, optional call output, and
                  optional error information.
         """
-        if cmd == Message.PING:
-            return (Message.PONG.value,)
+        if cmd == comm.Message.PING:
+            return (comm.Message.PONG.value,)
 
         try:
-            if cmd != Message.CALL:
+            if cmd != comm.Message.CALL:
                 raise ProtocolError(_('Unknown privsep cmd: %s') % cmd)
 
             # Extract the callable and arguments
@@ -447,14 +474,14 @@ class Daemon(object):
                 raise NameError(msg)
 
             ret = func(*f_args, **f_kwargs)
-            return (Message.RET.value, ret)
+            return (comm.Message.RET.value, ret)
         except Exception as e:
             LOG.debug(
                 'privsep: Exception during request[%(msgid)s]: '
                 '%(err)s', {'msgid': msgid, 'err': e}, exc_info=True)
             cls = e.__class__
             cls_name = '%s.%s' % (cls.__module__, cls.__name__)
-            return (Message.ERR.value, cls_name, e.args)
+            return (comm.Message.ERR.value, cls_name, e.args)
 
     def _create_done_callback(self, msgid):
         """Creates a future callback to receive command execution results.
@@ -482,11 +509,11 @@ class Daemon(object):
                     '%(err)s', {'msgid': msgid, 'err': e}, exc_info=True)
                 cls = e.__class__
                 cls_name = '%s.%s' % (cls.__module__, cls.__name__)
-                reply = (Message.ERR.value, cls_name, e.args)
+                reply = (comm.Message.ERR.value, cls_name, e.args)
                 try:
                     channel.send((msgid, reply))
-                except IOError:
-                    self.communication_error = sys.exc_info()
+                except IOError as exc:
+                    self.communication_error = exc
 
         return _call_back
 
@@ -501,10 +528,10 @@ class Daemon(object):
         for msgid, msg in self.channel:
             error = self.communication_error
             if error:
-                if error[1].errno == errno.EPIPE:
+                if error.errno == errno.EPIPE:
                     # Write stream closed, exit loop
                     break
-                six.reraise(*error)
+                raise error
 
             # Submit the command for execution
             future = self.thread_pool.submit(self._process_cmd, msgid, *msg)
@@ -524,7 +551,14 @@ def helper_main():
     logging.register_options(cfg.CONF)
 
     cfg.CONF(args=sys.argv[1:], project='privsep')
-    logging.setup(cfg.CONF, 'privsep')  # note replace_logging call below
+    # note replace_logging call below
+    try:
+        logging.setup(cfg.CONF, 'privsep', fix_eventlet=False)
+    except TypeError:
+        # NOTE(ralonsoh): in case of using oslo.log<5.0.2, kwarg
+        # "fix_eventlet" won't be defined. Remove this try clause when oslo.log
+        # is bumped.
+        logging.setup(cfg.CONF, 'privsep')
 
     context = importutils.import_class(cfg.CONF.privsep_context)
     from oslo_privsep import priv_context   # Avoid circular import
